'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _Constants = require('../../core/Constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Creates a new RouteController
 * @class
 */
class RouteController {

    /**
     * @prop {string} tab The tab to route in
     * @prop {Content} dataClass The base class of this route
     */
    constructor({ tab, dataClass }) {
        this.tab = tab;
        this.dataClass = dataClass;
    }

    /**
     * Get the number of pages in this tab
     * @returns {Promise<number>}
     */
    pages() {
        return this._request(`/${this.tab}s`).then(p => p.length);
    }

    /**
     * Searches in this route
     * @param {Object} [options={}] The search options
     * @prop {number} [options.page=1] The page to search
     * @prop {string} [options.sort='trending'] The way you want to sort<br/><br/>
     * Can be `name`, `rating`, `released`, `updated`, `trending` or `year`.
     * @prop {number} [options.order=-1] The order you want to sort
     * @prop {string} [options.genre='all'] The genre to filter
     * @prop {string} [options.query] The query to search
     * @returns {Promise<ContentList>}
     */
    search({ page = 1, sort = 'trending', order = -1, genre = 'all', query } = {}) {
        return this._request(`/${this.tab}s/${page}`, { sort, order, genre, keywords: query }).then(values => {
            return values.map(value => new this.dataClass(this, value));
        });
    }

    /**
     * Gets a random item of this route
     * @returns {Promise<Content>}
     */
    random() {
        return this._request(`/random/${this.tab}`).then(value => new this.dataClass(this, value));
    }

    /**
     * Gets the details of an item from its ID
     * @param {string} id The ID to get
     * @returns {Promise<Content>}
     */
    get(id) {
        return this._rawDetails(id).then(value => new this.dataClass(this, value));
    }

    /**
     * Gets the raw details from an ID
     * @param {string} id
     * @returns {Promise<Object>}
     * @private
     */
    _rawDetails(id) {
        if (typeof id === 'object') id = id.id;
        return this._request(`/${this.tab}/${id}`).then(data => {
            data.details = true;
            return data;
        });
    }

    /**
     * Requests to this route
     * @param {string} endpoint The endpoint
     * @param {Object} queryParams The parameters to the request
     * @returns {Promise<Object>}
     * @private
     */
    _request(endpoint, queryParams = {}) {
        queryParams = _querystring2.default.stringify(queryParams);
        return (0, _nodeFetch2.default)(`${_Constants.baseURL}${endpoint}?${queryParams}`).then(r => r.json());
    }
}

exports.default = RouteController; /**
                                    * Content classes.
                                    * @memberof RouteController
                                    * @typedef {Anime|Movie|Show} Content
                                    */

/**
 * Content classes list.
 * @memberof RouteController
 * @typedef {Content[]} ContentList
 */

module.exports = exports.default;